// Copyright 2024 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


import SwiftUI

import ArcGIS

private class Model: ObservableObject {

    let graphicsOverlay: GraphicsOverlay

    let locator = LocatorTask(
        url: URL(string: "https://geocode-api.arcgis.com/arcgis/rest/services/World/GeocodeServer")!
    )

    let textGraphic: Graphic = {
        let textSymbol = TextSymbol(
            text: "",
            color: .black,
            size: 14,
            horizontalAlignment: .center,
            verticalAlignment: .bottom
        )
        textSymbol.backgroundColor = .white
        return Graphic(symbol: textSymbol)
    }()

    let markerGraphic: Graphic = {
        let markerSymbol = SimpleMarkerSymbol(
            style: .square,
            color: .red,
            size: 14
        )
        return Graphic(symbol: markerSymbol)
    }()

    init() {
        graphicsOverlay = GraphicsOverlay(graphics: [textGraphic, markerGraphic])
    }

}

struct ContentView: View {

    @StateObject private var model = Model()

    @State private var map = Map(basemapStyle: .arcGISImagery)
    @State private var viewpoint: Viewpoint? = Viewpoint(
        latitude: 34.02700,
        longitude: -118.80500,
        scale: 72_000
    )

    @State private var searchText: String = ""

    private func geocode(with searchText: String) async throws {

        let parameters = GeocodeParameters()
        parameters.addResultAttributeName("*")
        parameters.maxResults = 1
        parameters.outputSpatialReference = map.spatialReference

        let geocodeResults = try await model.locator.geocode(forSearchText: searchText, using: parameters)
        if let firstResult = geocodeResults.first,
           let extent = firstResult.extent,
           let location = firstResult.displayLocation,
           let symbol = model.textGraphic.symbol as? TextSymbol {
            viewpoint = Viewpoint(boundingGeometry: extent)
            model.markerGraphic.geometry = location
            model.textGraphic.geometry = location
            symbol.text = firstResult.label
        }

    }

    var body: some View {

        MapView(map: map, viewpoint: viewpoint, graphicsOverlays: [model.graphicsOverlay])

            .overlay(alignment: .top) {

                HStack {
                    TextField("Enter address", text: $searchText)
                    Spacer()
                    Button("Search") {
                        Task {
                            try await geocode(with: searchText)
                        }
                    }
                }

                .padding(EdgeInsets(top: 60, leading: 10, bottom: 10, trailing: 10))
                .background(.thinMaterial, ignoresSafeAreaEdges: .horizontal)
            }

    }

}


